/*

     d888   8888888888     d88P 8888888888     d88P  .d8888b.  888888888
    d8888         d88P    d88P        d88P    d88P  d88P  Y88b 888
      888        d88P    d88P        d88P    d88P   888    888 888
      888       d88P    d88P        d88P    d88P    Y88b. d888 8888888b.
      888    88888888  d88P      88888888  d88P      "Y888P888      "Y88b
      888     d88P    d88P        d88P    d88P             888        888
      888    d88P    d88P        d88P    d88P       Y88b  d88P Y88b  d88P
    8888888 d88P    d88P        d88P    d88P         "Y8888P"   "Y8888P"

   []-----------------------------------------------------------------------[]
   |                             INTERSUP.C                                  |
   |                                                                         |
   |      Desenha a interseao de duas superfcies usando suas equaoes      |
   |  paramtricas f(u, v) = (f1(u, v), f2(u, v), f3(u, v))   e              |
   |               g(u, v) = (g1(u, v), g2(u, v), g3(u, v)).                 |
   |                                                                         |
   |                     por  Lenimar Nunes de Andrade                       |
   []-----------------------------------------------------------------------[]

*/

/* Antes da compilaao, escolha uma das seguintes opoes ou  entao
   digite as equaoes paramtricas das superfcies. Se for usado o
   TCC, digite uma linha conforme o exemplo a seguir:

   [C:\] TCC -Z -G -N -2 -DDOISCILINDROS -esup1 intersup.c graphics.lib
*/

/* #define DOISCILINDROS           */
/* #define CILINDROPARABOLOIDE     */
/* #define ESFERATUBO              */
/* #define DOISTOROS               */
/* #define SENOPARABOLOIDE         */
/* #define PLANOCILINDRO           */
/* #define PARABOLOIDECILINDRO     */
/* #define ELIPSOIDECONE           */
/* #define ESFERAPARABOLOIDE       */
/* #define ESFERAPLANO             */
/* #define HELICOIDEPARABOLOIDE    */
/* #define ESFERASELA              */
/* #define SELAPARABOLOIDE         */
/* #define SELAPLANO               */
/* #define CILINDROESFERA          */
/* #define PARABOLOIDESENORACIONAL */
/* #define PLANOSENORACIONAL       */
/* #define SENORACIONALCUBICO      */
/* #define PARABOLOIDECOSSENO      */
/* #define COSSENOXCOSSENOY        */
/* #define DOISCONES               */
/* #define CONEHELICOIDE           */
/* #define ASTROIDECILINDRO        */
/* #define PLANOSEN3XSEN2X         */
/* #define BEZIER1                 */
/* #define BEZIER2                 */
/* #define BEZIER3                 */
/* #define BEZIER4                 */
/* #define BEZIER5                 */

#include <math.h>
#include <graphics.h>
#include <conio.h>
#include <string.h>
#include <dos.h>
#include <stdio.h>

#define float long double

#define MAX_IT 100   /* Quant. maxima de iteracoes na resolucao do sistema */
#include "sis.c"
#define CAMINHO_BGI   ""
char mens1[80] = "", mens2[80] = "";
float altura = 0.0;
int pontos_descartados = 0;

/* ------------------------------------------------------------------------ */

/* Parametrizacoes das superfices
		  f(u, v) = (f1(u, v), f2(u, v), f3(u, v))
	    e     g(u, v) = (g1(u, v), g2(u, v), g3(u, v))         */

#ifdef DOISCILINDROS

  float nu = 30, nv = 30, inc = 0.05, em_baixo = 1;

  float f1(float u, float v) { return 4*cos(u); }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return 4*sin(u) + altura; }

  float g1(float u, float v) { return u; }
  float g2(float u, float v) { return 4*cos(v); }
  float g3(float u, float v) { return 4*sin(v); }

#endif

#ifdef CILINDROPARABOLOIDE

  float nu = 30, nv = 30, inc = 0.05, em_baixo = 0;

  float f1(float u, float v) { return v + 4*sin(u); }
  float f2(float u, float v) { return v ; }
  float f3(float u, float v) { return v + 4*cos(u) + altura; }

  float g1(float u, float v) { return u; }
  float g2(float u, float v) { return v; }
  float g3(float u, float v) { return 4 - u*u/5 - v*v/5; }

#endif

#ifdef ESFERATUBO

  float nu = 30, nv = 30, inc = 0.03, em_baixo = 0;

  float f1(float u, float v) { return (9 + 2*cos(u))*cos(v); }
  float f2(float u, float v) { return (9 + 2*cos(u))*sin(v); }
  float f3(float u, float v) { return 2*sin(u) + v + altura; }

  float g1(float u, float v) { return 9*cos(u)*sin(v); }
  float g2(float u, float v) { return 9*sin(u) - 10; }
  float g3(float u, float v) { return 9*cos(u)*cos(v); }

#endif

#ifdef DOISTOROS

  float nu = 35, nv = 35, inc = 0.04, em_baixo = 1;

  float f1(float u, float v) { return (12 + 2*cos(u))*cos(v); }
  float f2(float u, float v) { return (12 + 2*cos(u))*sin(v); }
  float f3(float u, float v) { return 2*sin(u) + altura; }

  float g1(float u, float v) { return (12 + 2*cos(u))*sin(v); }
  float g2(float u, float v) { return 2*sin(u); }
  float g3(float u, float v) { return (12 + 2*cos(u))*cos(v);; }

#endif

#ifdef SENOPARABOLOIDE

  float nu = 30, nv = 30, inc = 0.05, em_baixo = 1;

  float f1(float u, float v) { return u; }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return u*u + v*v + altura; }

  float g1(float u, float v) { return u; }
  float g2(float u, float v) { return v; }
  float g3(float u, float v) { return 8 + sin(v); }

#endif

#ifdef PLANOCILINDRO

  float nu = 30, nv = 30, inc = 0.05, em_baixo = 1;

  float f1(float u, float v) { return 4*cos(u); }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return 4*sin(u) + altura; }

  float g1(float u, float v) { return u; }
  float g2(float u, float v) { return v; }
  float g3(float u, float v) { return 4 - v - u/3; }

#endif

#ifdef PARABOLOIDECILINDRO

  float nu = 30, nv = 30, inc = 0.05, em_baixo = 1;

  float f1(float u, float v) { return 10 - u*u/4 - v*v/4; }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return u + altura; }

  float g1(float u, float v) { return 3*cos(u); }
  float g2(float u, float v) { return 3*sin(u); }
  float g3(float u, float v) { return v; }

#endif

#ifdef ELIPSOIDECONE

  float nu = 30, nv = 30, inc = 0.05, em_baixo = 1;

  float f1(float u, float v) { return 4*cos(u)*cos(v); }
  float f2(float u, float v) { return 7*cos(u)*sin(v) - 3; }
  float f3(float u, float v) { return 10*sin(u) + altura; }

  float g1(float u, float v) { return v*cos(u); }
  float g2(float u, float v) { return v; }
  float g3(float u, float v) { return v*sin(u); }

#endif

#ifdef ESFERAPARABOLOIDE

  float nu = 30, nv = 30, inc = 0.08, em_baixo = 1;

  float f1(float u, float v) { return 6*cos(u)*sin(v); }
  float f2(float u, float v) { return 6*cos(u)*cos(v); }
  float f3(float u, float v) { return 6*sin(u) + altura;}

  float g1(float u, float v) { return v; }
  float g2(float u, float v) { return u*u/10 + v*v/10; }
  float g3(float u, float v) { return u; }

#endif

#ifdef ESFERAPLANO

  float nu = 30, nv = 30, inc = 0.08, em_baixo = 1;

  float f1(float u, float v) { return 6*cos(u)*sin(v); }
  float f2(float u, float v) { return 6*cos(u)*cos(v); }
  float f3(float u, float v) { return 6*sin(u) + altura;}

  float g1(float u, float v) { return v; }
  float g2(float u, float v) { return 5 - u/10 + v; }
  float g3(float u, float v) { return u; }

#endif

#ifdef HELICOIDEPARABOLOIDE

  float nu = 30, nv = 30, inc = 0.06, em_baixo = 1;

  float f1(float u, float v) { return v*cos(u); }
  float f2(float u, float v) { return v*sin(u); }
  float f3(float u, float v) { return 2*u + altura;}

  float g1(float u, float v) { return u; }
  float g2(float u, float v) { return v*v - 10; }
  float g3(float u, float v) { return v; }

#endif

#ifdef ESFERASELA

  float nu = 30, nv = 30, inc = 0.07, em_baixo = 1;

  float f1(float u, float v) { return 6*cos(u)*sin(v); }
  float f2(float u, float v) { return 6*cos(u)*cos(v); }
  float f3(float u, float v) { return 6*sin(u) + altura;}

  float g1(float u, float v) { return v; }
  float g2(float u, float v) { return u*u/10 - v*v/10; }
  float g3(float u, float v) { return u; }

#endif

#ifdef SELAPARABOLOIDE

  float nu = 30, nv = 30, inc = 0.08, em_baixo = 1;

  float f1(float u, float v) { return u; }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return 12 - u*u/2 - v*v/2 + altura;}

  float g1(float u, float v) { return v; }
  float g2(float u, float v) { return u; }
  float g3(float u, float v) { return u*u/10 - v*v/10; }

#endif

#ifdef SELAPLANO

  float nu = 30, nv = 30, inc = 0.08, em_baixo = 1;

  float f1(float u, float v) { return u; }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return  1 + altura;}

  float g1(float u, float v) { return v; }
  float g2(float u, float v) { return u; }
  float g3(float u, float v) { return u*u/10 - v*v/10; }

#endif

#ifdef CILINDROESFERA

  float nu = 30, nv = 30, inc = 0.05, em_baixo = 0;

  float f1(float u, float v) { return 2*sin(u); }
  float f2(float u, float v) { return v ; }
  float f3(float u, float v) { return 2*cos(u) - 1 + altura; }

  float g1(float u, float v) { return 4*cos(u)*sin(v); }
  float g2(float u, float v) { return 4*cos(u)*cos(v); }
  float g3(float u, float v) { return 4*sin(u); }

#endif

#ifdef PARABOLOIDESENORACIONAL

  #define SEN(t)  ((2*(t))/(1 + (t)*(t)))

  float nu = 40, nv = 40, inc = 0.05, em_baixo = 1;

  float f1(float u, float v) { return u; }
  float f2(float u, float v) { return 2 - u*u/5 - v*v/5; }
  float f3(float u, float v) { return v + altura; }

  float g1(float u, float v) { return u; }
  float g2(float u, float v) { return v; }
  float g3(float u, float v) { return -3*SEN((v*v + u*u)/15); }

#endif

#ifdef PLANOSENORACIONAL

  #define SEN(t)  ((2*(t))/(1 + (t)*(t)))

  float nu = 40, nv = 40, inc = 0.05, em_baixo = 1;

  float f1(float u, float v) { return u; }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return altura; }

  float g1(float u, float v) { return u; }
  float g2(float u, float v) { return v; }
  float g3(float u, float v) { return 2 - 3*SEN((v*v + u*u)/15); }

#endif

#ifdef SENORACIONALCUBICO

  #define SEN(t)  ((2*(t))/(1 + (t)*(t)))

  float nu = 40, nv = 40, inc = 0.04, em_baixo = 1;

  float f1(float u, float v) { return v; }
  float f2(float u, float v) { return u; }
  float f3(float u, float v) { return (u*u*u - v*v*v)/120 + altura; }

  float g1(float u, float v) { return u; }
  float g2(float u, float v) { return v; }
  float g3(float u, float v) { return 2*SEN((u*u + v*v)/40) - 1; }

#endif

#ifdef BEZIER1

  float nu = 35, nv = 35, inc = 0.04, em_baixo = 0;

  #include "bezier.c"

  float f1(float u, float v) { return X_Bezier(p1, m, n, u, v); }
  float f2(float u, float v) { return Y_Bezier(p1, m, n, u, v); }
  float f3(float u, float v) { return Z_Bezier(p1, m, n, u, v) + altura; }

  float g1(float u, float v) { return u; }
  float g2(float u, float v) { return v; }
  float g3(float u, float v) { return -1 + u + v; }

#endif

#ifdef BEZIER2

  float nu = 35, nv = 35, inc = 0.02, em_baixo = 0;

  #include "bezier.c"

  float f1(float u, float v) { return X_Bezier(p1, m, n, u, v); }
  float f2(float u, float v) { return Y_Bezier(p1, m, n, u, v); }
  float f3(float u, float v) { return Z_Bezier(p1, m, n, u, v) + altura; }

  float g1(float u, float v) { return X_Bezier(p2, m, n, u, v); }
  float g2(float u, float v) { return Y_Bezier(p2, m, n, u, v); }
  float g3(float u, float v) { return Z_Bezier(p2, m, n, u, v); }

#endif

#ifdef BEZIER3

  float nu = 35, nv = 35, inc = 0.025, em_baixo = 0;

  #include "bezier.c"

  float f1(float u, float v) { return X_Bezier(p1, m, n, u, v); }
  float f2(float u, float v) { return Y_Bezier(p1, m, n, u, v); }
  float f3(float u, float v) { return Z_Bezier(p1, m, n, u, v) + altura; }

  float g1(float u, float v) { return X_Bezier(p2, m, n, u, v); }
  float g2(float u, float v) { return Y_Bezier(p2, m, n, u, v); }
  float g3(float u, float v) { return Z_Bezier(p2, m, n, u, v); }

#endif

#ifdef BEZIER4

  float nu = 35, nv = 35, inc = 0.02, em_baixo = 0;

  #include "bezier.c"

  float f1(float u, float v) { return X_Bezier(p1, m, n, u, v); }
  float f2(float u, float v) { return Y_Bezier(p1, m, n, u, v); }
  float f3(float u, float v) { return Z_Bezier(p1, m, n, u, v) + altura; }

  float g1(float u, float v) { return X_Bezier(p2, m, n, u, v); }
  float g2(float u, float v) { return Y_Bezier(p2, m, n, u, v); }
  float g3(float u, float v) { return Z_Bezier(p2, m, n, u, v); }

#endif

#ifdef BEZIER5

  float nu = 35, nv = 35, inc = 0.04, em_baixo = 1;

  #include "bezier.c"

  float f1(float u, float v) { return X_Bezier(p1, m, n, u, v); }
  float f2(float u, float v) { return Y_Bezier(p1, m, n, u, v); }
  float f3(float u, float v) { return Z_Bezier(p1, m, n, u, v) + altura; }

  float g1(float u, float v) { return 2*cos(u)*cos(v);}
  float g2(float u, float v) { return 2*cos(u)*sin(v); }
  float g3(float u, float v) { return 2*sin(u) + 3; }

#endif

#ifdef PARABOLOIDECOSSENO

  float nu = 40, nv = 40, inc = 0.03, em_baixo = 0;

  float f1(float u, float v) { return u; }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return -v*v/4 - u*u/4 + 10 + altura; }

  float g1(float u, float v) { return 4.5*cos(u); }
  float g2(float u, float v) { return v; }
  float g3(float u, float v) { return u; }

#endif

#ifdef COSSENOXCOSSENOY

  float nu = 35, nv = 35, inc = 0.035, em_baixo = 0;

  float f1(float u, float v) { return u*cos(v); }
  float f2(float u, float v) { return u; }
  float f3(float u, float v) { return u*sin(v) + altura; }

  float g1(float u, float v) { return v; }
  float g2(float u, float v) { return u; }
  float g3(float u, float v) { return 2*cos(0.5*v)*cos(0.5*u); }

#endif

#ifdef CONEHELICOIDE

  float nu = 30, nv = 30, inc = 0.02, em_baixo = 1;

  float f1(float u, float v) { return cos(u)*v; }
  float f2(float u, float v) { return sin(u)*v; }
  float f3(float u, float v) { return u + altura; }

  float g1(float u, float v) { return v*cos(u); }
  float g2(float u, float v) { return v*sin(u); }
  float g3(float u, float v) { return v; }

#endif

#ifdef DOISCONES

  float nu = 30, nv = 30, inc = 0.03, em_baixo = 1;

  float f1(float u, float v) { return cos(u)*v; }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return sin(u)*v + altura; }

  float g1(float u, float v) { return cos(u)*v; }
  float g2(float u, float v) { return sin(u)*v; }
  float g3(float u, float v) { return v; }

#endif

#ifdef ASTROIDECILINDRO

  float nu = 30, nv = 30, inc = 0.035, em_baixo = 1;

  float f1(float u, float v) { return cos(u)*5; }
  float f2(float u, float v) { return v ; }
  float f3(float u, float v) { return sin(u)*5 + altura; }

  float g1(float u, float v) { return v*cos(u)*cos(u)*cos(u); }
  float g2(float u, float v) { return v*sin(u)*sin(u)*sin(u); }
  float g3(float u, float v) { return v; }
#endif

/* ------------------------------------------------------------------------ */

#ifdef PLANOSEN3XSEN2X

  float nu = 50, nv = 50, inc = 0.03, em_baixo = 1;

  float f1(float u, float v) { return u; }
  float f2(float u, float v) { return v; }
  float f3(float u, float v) { return 6; }

  float g1(float u, float v) { return 1.5*v*sin(3*u); }
  float g2(float u, float v) { return 1.5*v*sin(2*u); }
  float g3(float u, float v) { return v; }

#endif

/* ------------------------------------------------------------------------ */

#include "graf.c"

float Const; /* Constante usada na construcao da intersecao */

float F1(float x, float y, float z) { return f1(x, y) - g1(Const, z); }
float F2(float x, float y, float z) { return f2(x, y) - g2(Const, z); }
float F3(float x, float y, float z) { return f3(x, y) - g3(Const, z); }

/* ------------------------------------------------------------------------ */

float G1(float x, float y, float z) { return f1(x, y) - g1(z, Const); }
float G2(float x, float y, float z) { return f2(x, y) - g2(z, Const); }
float G3(float x, float y, float z) { return f3(x, y) - g3(z, Const); }

/* ------------------------------------------------------------------------ */

float H1(float x, float y, float z) { return g1(x, y) - f1(Const, z); }
float H2(float x, float y, float z) { return g2(x, y) - f2(Const, z); }
float H3(float x, float y, float z) { return g3(x, y) - f3(Const, z); }

/* ------------------------------------------------------------------------ */

float I1(float x, float y, float z) { return g1(x, y) - f1(z, Const); }
float I2(float x, float y, float z) { return g2(x, y) - f2(z, Const); }
float I3(float x, float y, float z) { return g3(x, y) - f3(z, Const); }

/* ------------------------------------------------------------------------ */

void InterceptaSup(float (*f1) (float, float), float (*f2) (float, float),
		   float (*f3) (float, float), float (*g1) (float, float),
       float (*g2) (float, float), float (*g3) (float, float), float umin,
		float umax, float vmin, float vmax,
		float x0_aprox, float y0_aprox, float z0_aprox) {

  float u, v, x, y, z, x2, y2, z2;
  float x_0[3], sol[3], Erro = 1E-5;
  int n;

  for (u = umin; u <= umax; u += 2*inc) {
    Const = u;
    x_0[0] = x0_aprox; x_0[1] = y0_aprox; x_0[2] = z0_aprox;
    n = ResolveSistema(F1, F2, F3, x_0, 1E-8, MAX_IT, sol);
    if (n > 0 && n < MAX_IT) {
      x = f1(sol[0], sol[1]);
      y = f2(sol[0], sol[1]);
      z = f3(sol[0], sol[1]);
      x2 = g1(Const, sol[2]);
      y2 = g2(Const, sol[2]);
      z2 = g3(Const, sol[2]);
      if (DISTANCIA(x, y, z, x2, y2, z2) <= Erro)
	DesenhaPonto(x, y, z);
      else pontos_descartados++;
    }
  }

  for (u = umin; u <= umax; u += 2*inc) {
    Const = u;
    x_0[0] = x0_aprox; x_0[1] = y0_aprox; x_0[2] = z0_aprox;
    n = ResolveSistema(H1, H2, H3, x_0, 1E-8, MAX_IT, sol);
    if (n > 0 && n < MAX_IT) {
      x = g1(sol[0], sol[1]);
      y = g2(sol[0], sol[1]);
      z = g3(sol[0], sol[1]);
      x2 = f1(Const, sol[2]);
      y2 = f2(Const, sol[2]);
      z2 = f3(Const, sol[2]);
      if (DISTANCIA(x, y, z, x2, y2, z2) <= Erro)
	DesenhaPonto(x, y, z);
      else pontos_descartados++;
    }
  }

  /* ---------------- */

  for (v = vmin; v <= vmax; v += 2*inc) {
    Const = v;
    x_0[0] = x0_aprox; x_0[1] = y0_aprox; x_0[2] = z0_aprox;
    n = ResolveSistema(G1, G2, G3, x_0, 1E-8, MAX_IT, sol);
    if (n > 0 && n < MAX_IT) {
      x = f1(sol[0], sol[1]);
      y = f2(sol[0], sol[1]);
      z = f3(sol[0], sol[1]);
      x2 = g1(sol[2], Const);
      y2 = g2(sol[2], Const);
      z2 = g3(sol[2], Const);
      if (DISTANCIA(x, y, z, x2, y2, z2) <= Erro)
	DesenhaPonto(x, y, z);
      else pontos_descartados++;
    }
  }

  for (v = vmin; v <= vmax; v += 2*inc) {
    Const = v;
    x_0[0] = x0_aprox; x_0[1] = y0_aprox; x_0[2] = z0_aprox;
    n = ResolveSistema(I1, I2, I3, x_0, 1E-8, MAX_IT, sol);
    if (n > 0 && n < MAX_IT) {
      x = g1(sol[0], sol[1]);
      y = g2(sol[0], sol[1]);
      z = g3(sol[0], sol[1]);
      x2 = f1(sol[2], Const);
      y2 = f2(sol[2], Const);
      z2 = f3(sol[2], Const);
      if (DISTANCIA(x, y, z, x2, y2, z2) <= Erro)
	DesenhaPonto(x, y, z);
      else pontos_descartados++;
    }
  }

}

/* ------------------------------------------------------------------------ */

void Som(void) {

/* Emite um som para sinalizar o final da construcao da intersecao */

  sound(3000);  delay(20);
  sound(6000);  delay(20);
  sound(8000);  delay(20);
  sound(9000);  delay(50);
  sound(11000); delay(200);
  nosound();
}

/* ------------------------------------------------------------------------ */

void MensagemInicial(void) {

  printf("\n\n*** INTERSEAO ENTRE DUAS SUPERFICIES ***");
  printf("\n\n  ESC: encerra o programa");
  printf("\n  PgUp: aproxima o grfico         PgDn: afasta o grfico");
  printf("\n  Teclas de setas: movimentam a origem (0, 0, 0) na tela");
  printf("\n  F1: mostra os eixos");
  printf("\n  F2: mostra as equaoes das superfcies");
  printf("\n  F3: muda cor da superfcie 1");
  printf("\n  F4: muda cor da superfcie 2");
  printf("\n  F5: muda cor da curva-interseao");
  printf("\n  Ctrl-Fn: desloca n unidades para baixo a superfcie 1");
  printf("\n  Alt-Fn: desloca n unidades para cima a superfcie 1");
  printf("\n\nPressione qualquer tecla para continuar\n\n");
  getch();
}

/* ------------------------------------------------------------------------ */

void main(void) {

  int gd, gm, cor1 = BLUE, cor2 = RED, cor3 = YELLOW,
      eixos = 0, mostra_equacoes = 1;
  unsigned char ch, texto_aux[80];

  MensagemInicial();

  registerbgidriver(EGAVGA_driver);
  gd = DETECT;
  initgraph(&gd, &gm, CAMINHO_BGI);
  a = getmaxx()/2; b = getmaxy()/2;

  #ifdef DOISCILINDROS
    strcpy(mens1, "f(u, v) = (4 cos(u), v, 4 sen(u))");
    strcpy(mens2, "g(u, v) = (u, 4 cos(v), 4 sen(v))");
  #endif

  #ifdef CILINDROPARABOLOIDE
    strcpy(mens1, "f(u, v) = (v + 4 sen(u), v, v + 4 cos(u))");
    strcpy(mens2, "g(u, v) = (u, v, 4 - u^2/5 - v^2/5)");
  #endif

  #ifdef ESFERATUBO
    strcpy(mens1,
   "f(u, v) = ((9 + 2 cos(u)) cos(v), (9 + 2 cos(u)) sen(v), 2 sen(u) + v)");
    strcpy(mens2,
   "g(u, v) = (9 cos(u) sen(v), 9 sen(u) - 10, 9 cos(u) cos(v))");
  #endif

  #ifdef DOISTOROS
    teta = M_PI/8.0;
    strcpy(mens1,
   "f(u, v) = (12 + 2 cos(u)) cos(v), (12 + 2 cos(u)) sen(v), 2 sen(u))");
    strcpy(mens2,
   "g(u, v) = (12 + 2 cos(u)) sen(v), 2 sen(u), (12 + 2 cos(u)) cos(v))");
  #endif

  #ifdef SENOPARABOLOIDE
    eixos = 1; tam = 30; b = 3*getmaxy()/4;
    strcpy(mens1, "f(u, v) = (u, v, u^2 + v^2)");
    strcpy(mens2, "g(u, v) = (u, v, 8 + sen(v))");
  #endif

  #ifdef PLANOCILINDRO
    strcpy(mens1, "f(u, v) = (4 cos(u), v, 4 sen(u))");
    strcpy(mens2, "g(u, v) = (u, v, 4 - v - u/3)");
  #endif

  #ifdef PARABOLOIDECILINDRO
    strcpy(mens1, "f(u, v) = (10 - u^2/4 - v^2/4, v, u)");
    strcpy(mens2, "g(u, v) = (3 cos(u), 3 sen(u), v)");
  #endif

  #ifdef ELIPSOIDECONE
    strcpy(mens1, "f(u, v) = (4 cos(u) cos(v), 7 cos(u) sen(v) - 3, 10 sen(u))");
    strcpy(mens2, "g(u, v) = (v cos(u), v, v sen(u))");
  #endif

  #ifdef ESFERAPARABOLOIDE
    strcpy(mens1, "f(u, v) = (6 cos(u) sen(v), 6 cos(u) cos(v), 6 sen(u))");
    strcpy(mens2, "g(u, v) = (v, u^2/10 + v^2/10, u)");
  #endif

  #ifdef ESFERAPLANO
    strcpy(mens1, "f(u, v) = (6 cos(u) sen(v), 6 cos(u) cos(v), 6 sen(u))");
    strcpy(mens2, "g(u, v) = (v, 5 - u/10 + v, u)");
  #endif

  #ifdef HELICOIDEPARABOLOIDE
    strcpy(mens1, "f(u, v) = (v cos(u), v sen(u), 2 u)");
    strcpy(mens2, "g(u, v) = (u, v^2 - 10, v)");
  #endif

  #ifdef ESFERASELA
    strcpy(mens1, "f(u, v) = (6 cos(u) sen(v), 6 cos(u) cos(v), 6 sen(u))");
    strcpy(mens2, "g(u, v) = (v, u^2/10 - v^2/10, u)");
  #endif

  #ifdef SELAPARABOLOIDE
    strcpy(mens1, "f(u, v) = (u, v, 12 - u^2/2 - v^2/2)");
    strcpy(mens2, "g(u, v) = (v, u, u^2/10 - v^2/10)");
  #endif

  #ifdef SELAPLANO
    strcpy(mens1, "f(u, v) = (u, v, 1)");
    strcpy(mens2, "g(u, v) = (v, u, u^2/10 - v^2/10)");
  #endif

  #ifdef CILINDROESFERA
    tam = 30;
    strcpy(mens1, "f(u, v) = (2 sen(u), v, 2 cos(u) - 1)");
    strcpy(mens2, "g(u, v) = (4 cos(u) sen(v), 4 cos(u) cos(v), 4 sen(u))");
  #endif

  #ifdef PARABOLOIDESENORACIONAL
    tam = 25;
    strcpy(mens1, "f(u, v) = (u, 2 - u^2/5 - v^2/5, v)");
    strcpy(mens2, "g(u, v) = (u, v, -3 S((u^2 + v^2)/15)), S(x) = 2x/(1 + x^2)");
  #endif

  #ifdef PLANOSENORACIONAL
    tam = 35;
    strcpy(mens1, "f(u, v) = (u, v, 0)");
    strcpy(mens2, "g(u, v) = (u, v, 2 - 3 S((u^2 + v^2)/15)), S(x) = 2x/(1 + x^2)");
  #endif

  #ifdef SENORACIONALCUBICO
    tam = 30;
    strcpy(mens1, "f(u, v) = (v, u, (u^3 - v^3)/120)");
    strcpy(mens2, "g(u, v) = (u, v, 2 S((u^2 + v^2)/40) - 1), S(x) = 2x/(1 + x^2)");
  #endif

  #ifdef BEZIER1
    tam = 40; eixos = 1;
    strcpy(mens1, "f: superficie de Bezier (m = n = 2)");
    strcpy(mens2, "g(u, v) = (u, v, -1 + u + v)");

    m = 2;
    n = 2;

    DEF_PONTO(p1, 0, 0, -5, -5, 4);
    DEF_PONTO(p1, 0, 1, -5, 0, 0);
    DEF_PONTO(p1, 0, 2, -5, 5, -6);
    DEF_PONTO(p1, 1, 0, 0, -5, -10);
    DEF_PONTO(p1, 1, 1, 0, 0, 0);
    DEF_PONTO(p1, 1, 2, 0, 5, 6);
    DEF_PONTO(p1, 2, 0, 5, -5, 5);
    DEF_PONTO(p1, 2, 1, 5, 0, 0);
    DEF_PONTO(p1, 2, 2, 5, 5, -5);

  #endif

  #ifdef BEZIER2
    tam = 40; eixos = 1; b = 3*getmaxy()/4;
    strcpy(mens1, "f, g: superficies de Bezier");
    strcpy(mens2, "m = n = 2");

    m = 2;
    n = 2;

    DEF_PONTO(p1, 0, 0, -5, -5, 1);
    DEF_PONTO(p1, 0, 1, -5, 0, 2);
    DEF_PONTO(p1, 0, 2, -5, 5, 1);
    DEF_PONTO(p1, 1, 0, 0, -5, 1);
    DEF_PONTO(p1, 1, 1, 0, 0, 13);
    DEF_PONTO(p1, 1, 2, 0, 5, 5);
    DEF_PONTO(p1, 2, 0, 5, -5, 1);
    DEF_PONTO(p1, 2, 1, 5, 0, 1);
    DEF_PONTO(p1, 2, 2, 5, 5, -1);

    DEF_PONTO(p2, 0, 0, -5, -5, 5);
    DEF_PONTO(p2, 0, 1, -5, 0, 3);
    DEF_PONTO(p2, 0, 2, -5, 5, 1);
    DEF_PONTO(p2, 1, 0, 0, -5, 1);
    DEF_PONTO(p2, 1, 1, 0, 0, 3);
    DEF_PONTO(p2, 1, 2, 0, 5, 1);
    DEF_PONTO(p2, 2, 0, 5, -5, 1);
    DEF_PONTO(p2, 2, 1, 5, 0, 3);
    DEF_PONTO(p2, 2, 2, 5, 5, 5);

  #endif

  #ifdef BEZIER3

    tam = 70; eixos = 1; b = 3*getmaxy()/4; a = 3*getmaxx()/4;
    strcpy(mens1, "f, g: superficies de Bezier");
    strcpy(mens2, "m = n = 3");

    m = 3;
    n = 3;

    DEF_PONTO(p1, 0, 0, -5, -5, 1);
    DEF_PONTO(p1, 0, 1, -5, -3, 1);
    DEF_PONTO(p1, 0, 2, -5, -1, 2);
    DEF_PONTO(p1, 0, 3, -5,  0, 2);
    DEF_PONTO(p1, 1, 0, -3, -5, 3);
    DEF_PONTO(p1, 1, 1, -3, -3, 3);
    DEF_PONTO(p1, 1, 2, -3, -1, 2);
    DEF_PONTO(p1, 1, 3, -3,  0, 1);
    DEF_PONTO(p1, 2, 0, -1, -5, 0);
    DEF_PONTO(p1, 2, 1, -1, -3, 1);
    DEF_PONTO(p1, 2, 2, -1, -1, 1);
    DEF_PONTO(p1, 2, 3, -1,  0, 2);
    DEF_PONTO(p1, 3, 0,  0, -5, 1);
    DEF_PONTO(p1, 3, 1,  0, -3, -1);
    DEF_PONTO(p1, 3, 2,  0, -1, 0);
    DEF_PONTO(p1, 3, 3,  0,  0, 1);

    DEF_PONTO(p2, 0, 0, -5, -5, 2);
    DEF_PONTO(p2, 0, 1, -5, -3, 1);
    DEF_PONTO(p2, 0, 2, -5, -1, 2);
    DEF_PONTO(p2, 0, 3, -5,  0, 2);
    DEF_PONTO(p2, 1, 0, -3, -5, 1);
    DEF_PONTO(p2, 1, 1, -3, -3, 2);
    DEF_PONTO(p2, 1, 2, -3, -1, 3);
    DEF_PONTO(p2, 1, 3, -3,  0, 2);
    DEF_PONTO(p2, 2, 0, -1, -5, 2);
    DEF_PONTO(p2, 2, 1, -1, -3, 0);
    DEF_PONTO(p2, 2, 2, -1, -1, -1);
    DEF_PONTO(p2, 2, 3, -1,  0, 0);
    DEF_PONTO(p2, 3, 0,  0, -5, 1);
    DEF_PONTO(p2, 3, 1,  0, -3, 2);
    DEF_PONTO(p2, 3, 2,  0, -1, 1);
    DEF_PONTO(p2, 3, 3,  0,  0, 0);

  #endif

  #ifdef BEZIER4
    tam = 40; eixos = 1;
    strcpy(mens1, "f, g: superficies de Bezier");
    strcpy(mens2, "m = n = 2");

    m = 2;
    n = 2;

    DEF_PONTO(p1, 0, 0, -5, -5, 1);
    DEF_PONTO(p1, 0, 1, -5, 0, 1);
    DEF_PONTO(p1, 0, 2, -5, 5, 1);
    DEF_PONTO(p1, 1, 0, 0, -5, 1);
    DEF_PONTO(p1, 1, 1, 0, 0, 11);
    DEF_PONTO(p1, 1, 2, 0, 5, 1);
    DEF_PONTO(p1, 2, 0, 5, -5, 1);
    DEF_PONTO(p1, 2, 1, 5, 0, 1);
    DEF_PONTO(p1, 2, 2, 5, 5, 1);

    DEF_PONTO(p2, 0, 0, -5, -5, 1);
    DEF_PONTO(p2, 0, 1, -5, 0, 4);
    DEF_PONTO(p2, 0, 2, -5, 5, 3);
    DEF_PONTO(p2, 1, 0, 0, -5, 5);
    DEF_PONTO(p2, 1, 1, 0, 0, -2);
    DEF_PONTO(p2, 1, 2, 0, 5, -1);
    DEF_PONTO(p2, 2, 0, 5, -5, -2);
    DEF_PONTO(p2, 2, 1, 5, 0, -1);
    DEF_PONTO(p2, 2, 2, 5, 5, 4);

  #endif

  #ifdef BEZIER5
    tam = 45; eixos = 0; b = 3*getmaxy()/4;
    strcpy(mens1, "f: superficie de Bezier (m = n = 2)");
    strcpy(mens2, "g(u, v) = (2 cos(u) cos(v), 2 cos(u) sen(v), 2 sin(u) + 3)");

    m = 2;
    n = 2;

    DEF_PONTO(p1, 0, 0, -5, -5, 4);
    DEF_PONTO(p1, 0, 1, -5, 0, 1);
    DEF_PONTO(p1, 0, 2, -5, 5, 3);
    DEF_PONTO(p1, 1, 0, 0, -5, 1);
    DEF_PONTO(p1, 1, 1, 0, 0, 11);
    DEF_PONTO(p1, 1, 2, 0, 5, 1);
    DEF_PONTO(p1, 2, 0, 5, -5, 0);
    DEF_PONTO(p1, 2, 1, 5, 0, -1);
    DEF_PONTO(p1, 2, 2, 5, 5, 4);

  #endif

  #ifdef PARABOLOIDECOSSENO
    strcpy(mens1, "f(u, v) = (u, v, -v^2/4 -u^2/4 + 10)");
    strcpy(mens2, "g(u, v) = (9/2 cos(u), v, u)");
  #endif

  #ifdef COSSENOXCOSSENOY
    strcpy(mens1, "f(u, v) = (u, v, -v^2 - u^2/4 + 10)");
    strcpy(mens2, "g(u, v) = (v, u, 2 cos(v/2) cos(u/2))");
  #endif

  #ifdef CONEHELICOIDE
    strcpy(mens1, "f(u, v) = (v cos(u), v sen(u), u)");
    strcpy(mens2, "g(u, v) = (v cos(u), v sen(u), v)");
  #endif

  #ifdef DOISCONES
    strcpy(mens1, "f(u, v) = (v cos(u), v, v sen(u))");
    strcpy(mens2, "g(u, v) = (v cos(u), v sen(u), v)");
  #endif

  #ifdef ASTROIDECILINDRO
    strcpy(mens1, "f(u, v) = (5 cos(u), v, 5 sen(u))");
    strcpy(mens2, "g(u, v) = (v cos(u)^3, v sen(u)^3, v)");
  #endif

  #ifdef PLANOSEN3XSEN2X
    strcpy(mens1, "f(u, v) = (u, v, 6)");
    strcpy(mens2, "g(u, v) = (3/2 v sen(3 u), 3/2 v sen (2 u), v)");
  #endif

  do {

    #if defined(BEZIER1)
      setcolor(cor1);
      DesenhaSuperficie(f1, f2, f3, 0, 1, 0, 1);
      setcolor(cor2);
      DesenhaSuperficie(g1, g2, g3, -5, 5, -5, 5);
      setcolor(cor3);
      pontos_descartados = 0;
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.7, 0.8, 0.9);
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.2, 0.1, 0.0);
    #elif defined(BEZIER2)
      setcolor(cor1);
      DesenhaSuperficie(f1, f2, f3, 0, 1, 0, 1);
      setcolor(cor2);
      DesenhaSuperficie(g1, g2, g3, 0, 1, 0, 1);
      setcolor(cor3);
      pontos_descartados = 0;
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.7, 0.8, 0.9);
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.2, 0.1, 0.0);
    #elif defined(BEZIER3)
      setcolor(cor1);
      DesenhaSuperficie(f1, f2, f3, 0, 1, 0, 1);
      setcolor(cor2);
      DesenhaSuperficie(g1, g2, g3, 0, 1, 0, 1);
      setcolor(cor3);
      pontos_descartados = 0;
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.7, 0.8, 0.9);
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.2, 0.1, 0.0);

    #elif defined(BEZIER4)
      setcolor(cor1);
      DesenhaSuperficie(f1, f2, f3, 0, 1, 0, 1);
      setcolor(cor2);
      DesenhaSuperficie(g1, g2, g3, 0, 1, 0, 1);
      setcolor(cor3);
      pontos_descartados = 0;
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.7, 0.8, 0.9);
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.2, 0.1, 0.0);
    #elif defined(BEZIER5)
      setcolor(cor1);
      DesenhaSuperficie(f1, f2, f3, 0, 1, 0, 1);
      setcolor(cor2);
      DesenhaSuperficie(g1, g2, g3, -5, 5, -5, 5);
      setcolor(cor3);
      pontos_descartados = 0;
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.7, 0.8, 0.9);
      InterceptaSup(f1, f2, f3, g1, g2, g3, -5, 5, -5, 5,
		    0.2, 0.1, 0.0);
    #else
      setcolor(cor1);
      DesenhaSuperficie(f1, f2, f3, -8, 8, -8, 8);
      setcolor(cor2);
      DesenhaSuperficie(g1, g2, g3, -8, 8, -8, 8);
      setcolor(cor3);
      pontos_descartados = 0;
      InterceptaSup(f1, f2, f3, g1, g2, g3, -8, 8, -8, 8,
		    1.0, 1.0, 1.0);
      InterceptaSup(f1, f2, f3, g1, g2, g3, -8, 8, -8, 8,
		    -1.0, -1.0, -1.0);
    #endif

    if (mostra_equacoes) {
      setcolor(cor1);
      if (altura > 0)
	sprintf(texto_aux, " [ + %0.0Lf]", altura);
      else if (altura < 0)
	sprintf(texto_aux, " [ - %0.0Lf]", -altura);
      if (em_baixo) {
	outtextxy(1, getmaxy() - 4*textheight("("), mens1);
	if (altura != 0)
	  outtextxy(9*getmaxx()/10, getmaxy() - 4*textheight("("), texto_aux);
      }
      else {
	outtextxy(1, 2*textheight("("), mens1);
	if (altura != 0)
	  outtextxy(9*getmaxx()/10, 2*textheight("("), texto_aux);
      }
      setcolor(cor2);
      if (em_baixo)
	outtextxy(1, getmaxy() - 2*textheight("("), mens2);
      else
	outtextxy(1, 4*textheight("("), mens2);
    }
    if (eixos) DesenhaEixos();

    Som();
    ch = getch();
    if (ch == 0) ch = getch();
    switch (ch) {
      case SetaPCima: if (b > 40) b -= 40; break;
      case SetaPBaixo: b += 40; break;
      case SetaPDireita:  a += 40; break;
      case SetaPEsquerda: if (a > 40) a -= 40; break;
      case PgUp: tam += 5; break;
      case PgDn: if (tam > 5) tam -= 5; break;
      case TeclaF1: eixos ^= 1; break;
      case TeclaF2: mostra_equacoes ^= 1; break;
      case TeclaF3: cor1 = (cor1 + 1) % 16; break;
      case TeclaF4: cor2 = (cor2 + 1) % 16; break;
      case TeclaF5: cor3 = (cor3 + 1) % 16; break;
      case CtrlSetaPCima:  altura++; break;
      case CtrlSetaPBaixo: altura--; break;
      case CtrlF1: altura -= 1; break;
      case AltF1: altura += 1; break;
      case CtrlF2: altura -= 2; break;
      case AltF2: altura += 2; break;
      case CtrlF3: altura -= 3; break;
      case AltF3: altura += 3; break;
      case CtrlF4: altura -= 4; break;
      case AltF4: altura += 4; break;
      case CtrlF5: altura -= 5; break;
      case AltF5: altura += 5; break;
      case CtrlF6: altura -= 6; break;
      case AltF6: altura += 6; break;
      case CtrlF7: altura -= 7; break;
      case AltF7: altura += 7; break;
      case CtrlF8: altura -= 8; break;
      case AltF8: altura += 8; break;
      case CtrlF9: altura -= 9; break;
      case AltF9: altura += 9; break;
      case CtrlF10: altura -= 10; break;
      case AltF10: altura += 10; break;
    }
    cleardevice();
  } while (ch != ESC);

  closegraph();
  if (pontos_descartados)
    printf("\nForam descartados %d pontos da interseao\n", pontos_descartados);
}

/*** FIM DE INTERSUP.C ***/

