/*

     d888   8888888888     d88P 8888888888     d88P  .d8888b.  888888888
    d8888         d88P    d88P        d88P    d88P  d88P  Y88b 888
      888        d88P    d88P        d88P    d88P   888    888 888
      888       d88P    d88P        d88P    d88P    Y88b. d888 8888888b.
      888    88888888  d88P      88888888  d88P      "Y888P888      "Y88b
      888     d88P    d88P        d88P    d88P             888        888
      888    d88P    d88P        d88P    d88P       Y88b  d88P Y88b  d88P
    8888888 d88P    d88P        d88P    d88P         "Y8888P"   "Y8888P"

*/

/*** SIS.C ***/

/* ----------------------------------------------------------------------- */

int Inversa(float m[3][3]) {

/* Calcula a matriz inversa de m 3 x 3. Devolve 0 se a matriz nao tiver
   inversa ou 1 se a inversa existir. */

  float Det_M, a, b, c, d, e, f, g, h, i;
  const float epsilon = 1E-10;

  a = m[0][0]; b = m[0][1]; c = m[0][2];
  d = m[1][0]; e = m[1][1]; f = m[1][2];
  g = m[2][0]; h = m[2][1]; i = m[2][2];

  Det_M = b*f*g + c*d*h + a*e*i - c*e*g - a*f*h - b*d*i;

  if (fabs(Det_M) <= epsilon) return 0;

  m[0][0] = (e*i - f*h)/Det_M;
  m[0][1] = (c*h - b*i)/Det_M;
  m[0][2] = (b*f - c*e)/Det_M;

  m[1][0] = (f*g - d*i)/Det_M;
  m[1][1] = (a*i - c*g)/Det_M;
  m[1][2] = (c*d - a*f)/Det_M;

  m[2][0] = (d*h - e*g)/Det_M;
  m[2][1] = (b*g - a*h)/Det_M;
  m[2][2] = (a*e - b*d)/Det_M;

  return 1;
}

/* ----------------------------------------------------------------------- */

void MatrizJacobiana(float m[3][3],  float (*f1)(float, float, float),
     float (*f2)(float, float, float), float (*f3)(float, float, float),
							  float x[3]) {

/* Calcula a matriz jacobina J   = (df /dx )
			      ij      i   j 3x3    */

  const float h = 1E-6;

  m[0][0] = (f1(x[0] + h, x[1], x[2]) - f1(x[0] - h, x[1], x[2]))/(2.0*h);
  m[0][1] = (f1(x[0], x[1] + h, x[2]) - f1(x[0], x[1] - h, x[2]))/(2.0*h);
  m[0][2] = (f1(x[0], x[1], x[2] + h) - f1(x[0], x[1], x[2] - h))/(2.0*h);

  m[1][0] = (f2(x[0] + h, x[1], x[2]) - f2(x[0] - h, x[1], x[2]))/(2.0*h);
  m[1][1] = (f2(x[0], x[1] + h, x[2]) - f2(x[0], x[1] - h, x[2]))/(2.0*h);
  m[1][2] = (f2(x[0], x[1], x[2] + h) - f2(x[0], x[1], x[2] - h))/(2.0*h);

  m[2][0] = (f3(x[0] + h, x[1], x[2]) - f3(x[0] - h, x[1], x[2]))/(2.0*h);
  m[2][1] = (f3(x[0], x[1] + h, x[2]) - f3(x[0], x[1] - h, x[2]))/(2.0*h);
  m[2][2] = (f3(x[0], x[1], x[2] + h) - f3(x[0], x[1], x[2] - h))/(2.0*h);
}

/* ----------------------------------------------------------------------- */

void MatrizVezesVetor(float m[3][3], float v[3], float prod[3]) {

/* Calcula prod[] = m[][] . v[] */

  prod[0] = m[0][0]*v[0] + m[0][1]*v[1] + m[0][2]*v[2];
  prod[1] = m[1][0]*v[0] + m[1][1]*v[1] + m[1][2]*v[2];
  prod[2] = m[2][0]*v[0] + m[2][1]*v[1] + m[2][2]*v[2];
}

/* ----------------------------------------------------------------------- */

void SubtraiVetores(float v1[3], float v2[3], float dif[3]) {

/* Calcula dif[] = v1[] - v2[] */

  dif[0] = v1[0] - v2[0];
  dif[1] = v1[1] - v2[1];
  dif[2] = v1[2] - v2[2];
}

/* ----------------------------------------------------------------------- */

void CalculaVetorF(float v[3], float (*f)(float, float, float),
	   float (*g)(float, float, float), float (*h)(float, float, float),
	   float x[3]) {

/* Calcula o vetor v[] = (f(x[]), g(x[]), h(x[])) */

  v[0] = f(x[0], x[1], x[2]);
  v[1] = g(x[0], x[1], x[2]);
  v[2] = h(x[0], x[1], x[2]);
}
/* ----------------------------------------------------------------------- */

float CalculaErro(float x[3], float y[3]) {

/* Calcula a distancia entre x[] e y[] */

#define QUAD(x) ((x)*(x))

  return sqrt(QUAD(x[0] - y[0]) + QUAD(x[1] - y[1]) + QUAD(x[2] - y[2]));
}

/* ----------------------------------------------------------------------- */

/* Iguala o vetor x[] a y[]; e' o mesmo que memcpy(x, y, 3*sizeof(float)) */

#define IGUALAVETOR(x, y)  x[0] = y[0]; x[1] = y[1]; x[2] = y[2]

/* ----------------------------------------------------------------------- */

int ResolveSistema(float (*f)(float, float, float),
		   float (*g)(float, float, float),
		   float (*h)(float, float, float),
		   float x_inicial[3],
		   float erro,
		   int max_iteracoes,
		   float solucao[3]) {

/* Resolve o sistema de 3 equacoes
			   
			     f(x, y, z) = 0
			     g(x, y, z) = 0
			     h(x, y, z) = 0
			   
  Se houver erro, e' retornado o valor -1; senao, e' retornado a quantidade
  de iteracoes utilizadas. Se encontrada, a solucao e' retornada no parame-
  tro solucao[].  Sao usados a aproximacao inicial da solucao  x_inicial[],
  e uma quantidade maxima de iteracoes max_iteracoes.

  Ex.: long double x_0[3] = {1.0, 1.0, 1.0}, X[3];
       if (ResolveSistema(F, G, H, x_0, 1E-10, 1000, X) < 0)
	 printf("Sistema nao pode ser resolvido");
       else
	 printf("Solucao encontrada = (%Lf, %Lf, %Lf)", X[0], X[1], X[2]);
*/

  float mat[3][3], prod[3], F[3], dif_vetores;
  int quant_iteracoes = 0;

  do {
    CalculaVetorF(F, f, g, h, x_inicial);
    MatrizJacobiana(mat, f, g, h, x_inicial);
    quant_iteracoes++;
    if (Inversa(mat)) {
      MatrizVezesVetor(mat, F, prod);
      SubtraiVetores(x_inicial, prod, solucao);
      dif_vetores = CalculaErro(x_inicial, solucao);
      IGUALAVETOR(x_inicial, solucao);
    }
    else {
      /* ERRO: Matriz jacobiana nao inversivel */
      return -1;
    }
  } while (dif_vetores > erro && quant_iteracoes < max_iteracoes);

  return quant_iteracoes;
}

/* ----------------------------------------------------------------------- */

/*** FIM DE SIS.C ***/
